import 'dart:ui';

import 'package:flutter/material.dart';

class ColorUtil {
  static Color getContrastColor(Color a, Color b) {
    double luminance(Color color) {
      return (0.299 * color.r + 0.587 * color.g + 0.114 * color.b) / 255;
    }

    double contrastRatio(Color a, Color b) {
      final lumA = luminance(a) + 0.05;
      final lumB = luminance(b) + 0.05;
      return lumA > lumB ? lumA / lumB : lumB / lumA;
    }

    if (contrastRatio(a, b) < 4.5) {
      // Find a color with higher contrast within the same hue
      final hsv = HSVColor.fromColor(a);
      double hue = hsv.hue;
      double saturation = hsv.saturation;
      double brightness = hsv.value;

      // Increase brightness to ensure higher contrast
      brightness = brightness > 0.5 ? brightness - 0.5 : brightness + 0.5;

      return HSVColor.fromAHSV(1.0, hue, saturation, brightness).toColor();
    }

    return b;
  }

  static Color getSelfContrastColor(Color color) {
    final hsv = HSVColor.fromColor(color);
    double hue = hsv.hue;
    double saturation = hsv.saturation;
    double brightness = hsv.value;

    // Increase brightness to ensure higher contrast
    brightness = brightness > 0.5 ? brightness - 0.5 : brightness + 0.5;

    return HSVColor.fromAHSV(1.0, hue, saturation, brightness).toColor();
  }
}

extension ColorUtilExtension on Color {
  Color get highlight => ColorUtil.getSelfContrastColor(this);
}
